/* -------------------------------------
 * Bar container
 * ------------------------------------- */
.progress-radial {
  float: left;
  position: relative;
  width: 100px;
  height: 100px;
  border-radius: 50%;
  border: 2px solid #000000;
  background-color: #75ad0a;
  left: 50%;
  margin-left: -50px;
  display: none; }
  .progress-radial .overlay {
    display: block;
    position: absolute;
    width: 90px;
    height: 90px;
    background-color: #ffffff;
    border-radius: 50%;
    margin-left: 5px;
    margin-top: 5px;
    text-align: center;
    line-height: 90px;
    font-size: 16px;
    padding: 0; }
  .progress-radial #tick {
    display: none;
    position: absolute;
    top: -2px;
    right: -2px;
    box-shadow: none;
    width: 80px;
    border-radius: 50%;
    background: white;
    border: 2px solid rgba(0, 0, 0, 0.2);
    padding: 10px; }
  .progress-radial #fail {
    display: none;
    position: absolute;
    top: -2px;
    right: -2px;
    box-shadow: none;
    width: 80px;
    height: 80px;
    border-radius: 50%;
    background: white;
    border: 2px solid rgba(0, 0, 0, 0.2);
    padding: 10px; }

#dropzone {
  display: none;
  background-color: rgba(0, 0, 0, 0.1);
  border-radius: 5px;
  margin: 20px;
  padding: 200px;
  position: relative;
  border: 3px dotted rgba(0, 0, 0, 0.3);
  text-align: center; }
  #dropzone.hover {
    box-shadow: 0 0px 10px 0px rgba(0, 0, 0, 0.5); }
  #dropzone.done #ftw-file-upload {
    display: none; }
  #dropzone.done.hover {
    box-shadow: 0 0px 10px 0px rgba(0, 0, 0, 0.5); }
  #dropzone.uploading {
    border: none; }
    #dropzone.uploading #ftw-file-upload {
      display: none; }
  #dropzone.fail #ftw-file-upload {
    display: none; }
  #dropzone #ftw-file-upload {
    display: inline;
    box-shadow: none;
    border: none;
    transition: transform 0.2s ease-in-out; }
  #dropzone img {
    border: 5px solid #FFFFFF;
    box-shadow: 0px 0px 0px 4px #000000;
    width: 100px; }
  #dropzone #dropzone-replacement-warning {
    color: darkred;
    text-align: center;
    margin-bottom: 40px;
    font-size: 14px; }

progress {
  width: 100%; }

progress:after {
  content: '%'; }

/*# sourceMappingURL=uploader.css.map */
