  @import 'progress';
  #dropzone {
    display: none;
    background-color: rgba(0, 0, 0, 0.1);
    border-radius: 5px;
    margin: 20px;
    padding: 200px;
    position: relative;
    border: 3px dotted rgba(0, 0, 0, 0.3);
    text-align: center;
    &.hover {
      box-shadow: 0 0px 10px 0px rgba(0, 0, 0, 0.5);
      #ftw-file-upload {
        transform: rotate(360deg);
      }
    }
    &.done {
      #ftw-file-upload {
        display: none;
      }
      &.hover {
        box-shadow: 0 0px 10px 0px rgba(0, 0, 0, 0.5);
      }
    }
    &.uploading {
      border: none;
      #ftw-file-upload {
        display: none;
      }
    }
    &.fail {
      #ftw-file-upload {
        display: none;
      }
    }
    #ftw-file-upload {
      display: inline;
      box-shadow: none;
      border: none;
      transition: transform 0.2s ease-in-out;
    }
    img {
      border: 5px solid #FFFFFF;
      box-shadow: 0px 0px 0px 4px #000000;
      width: 100px;
    }
    #dropzone-replacement-warning {
      color: darkred;
      text-align: center;
      margin-bottom: 40px;
      font-size: 14px;
    }
  }
  progress {
    width: 100%;
  }
  progress:after {
    content: '%';
  }
